@@ -148,8 +148,8 @@ def read_rnaplex(rnaplex, genes, genomes, features, srnas):
                     target_locus = "_".join(tags[:-3])
                     target_id = tags[-3]
                     detail = "_".join(tags[-2:])
-                    gene_id = get_gene_id(detail, target_id,
-                                          genes, genomes, features)
+                    gene_id, tar, target_id, target_locus = get_gene_id(
+                            detail, genes, genomes, features)
                 elif count_seq == 2:
                     srna = line[1:]
                     if srna not in srnas["RNAplex"].keys():
@@ -173,11 +173,9 @@ def read_rnaup(rnaup, srna_names, srnas, genes, genomes, features):
                     srna = line[1:]
                 else:
                     tags = line[1:].split("_")
-                    target_locus = "_".join(tags[:-3])
-                    target_id = tags[-3]
                     detail = "_".join(tags[-2:])
-                    gene_id = get_gene_id(detail, target_id, genes, genomes,
-                                          features)
+                    gene_id, tar, target_id, target_locus = get_gene_id(
+                            detail, genes, genomes, features)
                     if srna in srnas["RNAup"].keys():
                         srnas["RNAup"][srna].append({
                             "target_id": target_id, "target_locus": target_locus,
@@ -201,20 +199,20 @@ def read_intarna(intarna, srnas, genes, genomes, features):
                 if len(inter) == 9:
                     srna = inter[3]
                     tags = inter[0].split("_")
-                    target_locus = "_".join(tags[:-3])
-                    target_id = tags[-3]
-                    detail = "_".join(tags[-2:])
-                    if (len(tags[0])) != 0:
-                        gene_id = get_gene_id(detail, target_id, genes,
-                                              genomes, features)
-                        if srna not in srnas["IntaRNA"].keys():
-                            srnas["IntaRNA"][srna] = []
-                        srnas["IntaRNA"][srna].append({
-                            "target_id": target_id, "target_locus": target_locus,
-                            "detail": detail, "energy": float(inter[-1]),
-                            "gene_id": gene_id,
-                            "tar_pos": ",".join(inter[1:3]),
-                            "srna_pos": ",".join(inter[4:6])})
+                    if len(tags) >= 4:
+                        detail = "_".join(tags[-2:])
+                        if (len(tags[0])) != 0:
+                            gene_id, tar, target_id, target_locus = get_gene_id(
+                                    detail, genes, genomes, features)
+                            if tar is not None:
+                                if (srna not in srnas["IntaRNA"].keys()):
+                                    srnas["IntaRNA"][srna] = []
+                                srnas["IntaRNA"][srna].append({
+                                    "target_id": target_id, "target_locus": target_locus,
+                                    "detail": detail, "energy": float(inter[-1]),
+                                    "gene_id": gene_id,
+                                    "tar_pos": ",".join(inter[1:3]),
+                                    "srna_pos": ",".join(inter[4:6])})
 
 def read_table(gffs, rnaplex, rnaup, intarna, genes, genomes, features):
     srnas = {"RNAup": {}, "RNAplex": {}, "IntaRNA": {}}
@@ -231,56 +229,59 @@ def read_table(gffs, rnaplex, rnaup, intarna, genes, genomes, features):
     return srnas
 
 
-def get_gene_id(detail, tar_id, genes, gffs, features):
+def get_gene_id(detail, genes, gffs, features):
+    tar = None
     for gff in gffs:
         if gff.feature in features:
-            if tar_id != "NA":
-                if "ID" in gff.attributes.keys():
-                    if gff.attributes["ID"] == tar_id:
-                        tar = gff
-                        break
-            else:
-                start = int(detail.split("-")[0])
-                end = int(detail.split("-")[-1].split("_")[0])
-                strand = detail.split("_")[-1]
-                if (gff.start == start) and (gff.end == end) and (
-                        gff.strand == strand):
-                    tar = gff
-                    break
+            strand = detail.split("_")[-1]
+            start = int(detail.split("-")[0])
+            end = int(detail.split("-")[1].split("_")[0])
+            strand = detail.split("_")[-1]
+            if (gff.start == start) and (gff.end == end) and (
+                    gff.strand == strand):
+                tar = gff
+                break
     gene_id = "NA"
-    for gene in genes:
-        if "Parent" in tar.attributes.keys():
-            if "ID" in gene.attributes.keys():
-                if (gene.attributes["ID"] in
-                        tar.attributes["Parent"].split(",")):
-                    gene_id = gene.attributes["ID"]
-                    return gene_id
-        if gene_id == "NA":
-            if (gene.seq_id == tar.seq_id) and (
-                    gene.strand == tar.strand) and (
-                    (tar.start == gene.start) and (
-                    tar.end == gene.end)):
-                if "ID" in gene.attributes.keys():
-                    gene_id = gene.attributes["ID"]
-                    return gene_id
-    if gene_id == "NA":
+    target_id = "NA"
+    target_locus = "NA"
+    if tar is not None:
+        if "ID" in tar.attributes.keys():
+            target_id = tar.attributes["ID"]
+        if "locus_tag" in tar.attributes.keys():
+            target_locus = tar.attributes["locus_tag"]
         for gene in genes:
-            if (gene.seq_id == tar.seq_id) and (
-                    gene.strand == tar.strand):
-                if ((tar.start <= gene.start) and (
-                        tar.end >= gene.end)) or (
-                        (tar.start >= gene.start) and (
-                        tar.end <= gene.end)) or (
-                        (tar.start <= gene.start) and (
-                        tar.end <= gene.end) and (
-                        tar.end >= gene.start)) or (
-                        (tar.start >= gene.start) and (
-                        tar.start <= gene.end) and (
-                        tar.end >= gene.end)):
+            if "Parent" in tar.attributes.keys():
+                if "ID" in gene.attributes.keys():
+                    if (gene.attributes["ID"] in
+                            tar.attributes["Parent"].split(",")):
+                        gene_id = gene.attributes["ID"]
+                        return gene_id, tar, target_id, target_locus
+            if gene_id == "NA":
+                if (gene.seq_id == tar.seq_id) and (
+                        gene.strand == tar.strand) and (
+                        (tar.start == gene.start) and (
+                        tar.end == gene.end)):
                     if "ID" in gene.attributes.keys():
                         gene_id = gene.attributes["ID"]
-                        return gene_id
-    return gene_id
+                        return gene_id, tar, target_id, target_locus
+        if gene_id == "NA":
+            for gene in genes:
+                if (gene.seq_id == tar.seq_id) and (
+                        gene.strand == tar.strand):
+                    if ((tar.start <= gene.start) and (
+                            tar.end >= gene.end)) or (
+                            (tar.start >= gene.start) and (
+                            tar.end <= gene.end)) or (
+                            (tar.start <= gene.start) and (
+                            tar.end <= gene.end) and (
+                            tar.end >= gene.start)) or (
+                            (tar.start >= gene.start) and (
+                            tar.start <= gene.end) and (
+                            tar.end >= gene.end)):
+                        if "ID" in gene.attributes.keys():
+                            gene_id = gene.attributes["ID"]
+                            return gene_id, tar, target_id, target_locus
+    return gene_id, tar, target_id, target_locus
 
 
 def append_merge_three_methods(name, srna_info, ps_pos, pt_pos, u_data, i_data,
@@ -469,11 +470,12 @@ def merge_three(srnas, method3, srna, detect, srna_m1, top):
         for srna_m3 in srnas[method3][srna]:
             if (srna_m1["detail"] == srna_m3["detail"]) and (
                     "rank" in srna_m3.keys()):
-                if (srna_m3["rank"] <= top) and (
-                        "print" not in srna_m3.keys()) and (
-                        srna_m3["tar_pos"] != "NA"):
-                    srna_m3["print"] = True
-                    detect["3"] = True
+                if (srna_m3["rank"] != "NA"):
+                    if (srna_m3["rank"] <= top) and (
+                            "print" not in srna_m3.keys()) and (
+                            srna_m3["tar_pos"] != "NA"):
+                        srna_m3["print"] = True
+                        detect["3"] = True
                 return srna_m3
     else:
         return srna_m3
@@ -524,44 +526,46 @@ def merge_result(srnas, srna_gffs, args_tar, gffs, merges, length, methods):
         name = srna_datas[0]
         srna_info = srna_datas[1]
         for srna_m1 in srna_m1s:
-            if "rank" in srna_m1.keys():
-                if ("print" not in srna_m1.keys()) and (
-                        srna_m1["rank"] <= args_tar.top) and (
-                        srna_m1["tar_pos"] != "NA"):
-                    detect = {"2": False, "3": False}
-                    srna_m3 = None
-                    for srna_m2 in srnas[method2][srna]:
-                        if (srna_m1["detail"] == srna_m2["detail"]):
-                            if ("rank" in srna_m2.keys()):
-                                if (srna_m2["rank"] <= args_tar.top) and (
-                                        "print" not in srna_m2.keys()) and (
-                                         srna_m2["tar_pos"] != "NA"):
-                                    detect["2"] = True
-                                    srna_m3 = merge_three(srnas, method3, srna,
-                                                          detect, srna_m1,
-                                                          args_tar.top)
-                                    if (len(methods) == 2) or (
-                                            (len(methods) == 3) and detect["3"]):
-                                        target_info = get_target_info(
-                                            gffs, srna_m1)
-                                        import_merge(
-                                            overlaps, name, srna_info, srna_m1,
-                                            srna_m2, srna_m3, target_info,
-                                            args_tar.tar_start,
-                                            args_tar.tar_end, length,
-                                            len(methods))
-                                        srna_m1["print"] = True
-                                        srna_m2["print"] = True
-                                        import_merge(
-                                            merges, name, srna_info, srna_m1,
-                                            srna_m2, srna_m3, target_info,
-                                            args_tar.tar_start,
-                                            args_tar.tar_end, length,
-                                            len(methods))
-                            break
-                    check_non_overlap(detect, methods, srna_m1, srna_m2,
-                                      srna_m3, gffs, merges, name, srna_info,
-                                      args_tar, length, method3, srna, srnas)
+            if ("rank" in srna_m1.keys()):
+                if (srna_m1["rank"] != "NA"):
+                    if ("print" not in srna_m1.keys()) and (
+                            srna_m1["rank"] <= args_tar.top) and (
+                            srna_m1["tar_pos"] != "NA"):
+                        detect = {"2": False, "3": False}
+                        srna_m3 = None
+                        for srna_m2 in srnas[method2][srna]:
+                            if (srna_m1["detail"] == srna_m2["detail"]):
+                                if ("rank" in srna_m2.keys()):
+                                    if (srna_m2["rank"] != "NA"):
+                                        if (srna_m2["rank"] <= args_tar.top) and (
+                                                "print" not in srna_m2.keys()) and (
+                                                 srna_m2["tar_pos"] != "NA"):
+                                            detect["2"] = True
+                                            srna_m3 = merge_three(srnas, method3, srna,
+                                                                  detect, srna_m1,
+                                                                  args_tar.top)
+                                            if (len(methods) == 2) or (
+                                                    (len(methods) == 3) and detect["3"]):
+                                                target_info = get_target_info(
+                                                    gffs, srna_m1)
+                                                import_merge(
+                                                    overlaps, name, srna_info, srna_m1,
+                                                    srna_m2, srna_m3, target_info,
+                                                    args_tar.tar_start,
+                                                    args_tar.tar_end, length,
+                                                    len(methods))
+                                                srna_m1["print"] = True
+                                                srna_m2["print"] = True
+                                                import_merge(
+                                                    merges, name, srna_info, srna_m1,
+                                                    srna_m2, srna_m3, target_info,
+                                                    args_tar.tar_start,
+                                                    args_tar.tar_end, length,
+                                                    len(methods))
+                                break
+                        check_non_overlap(detect, methods, srna_m1, srna_m2,
+                                          srna_m3, gffs, merges, name, srna_info,
+                                          args_tar, length, method3, srna, srnas)
     return overlaps
 
 
